### This program uses dataset Age_Sex_Geno_Trait_data.csv to run the models of the paper ### 
### Fan RZ, Zhang YW, Albert PS, Liu AY, Wang YJ, and Xiong MM (2012) Longitudinal       ###
### association analysis of quantitative traits. Genetic Epidemiology 36:856-869.        ### 
### If you use the codes, please cite the above paper. Thanks.                           ###
############################################################################################
library(nlme)

### set up a log functions, which is used to generate dataset Age_Sex_Geno_Trait_data.csv ###
### Refer to simulation_log-mu_codes_for_Age_Sex_Geno_Trait_dat.R for details to generate the trait y ###
mut <- function(t)
   {return(-34.2+81.7*log(0.3*(t+21.7)) ) }

### Load data and setup covariates and genotype and trait data ###
Age_Sex_Geno_Trait_data = read.csv("C:/NICHD/Research/software/Fan/Longitudinal_qtl_popu_web/Age_Sex_Geno_Trait_data.csv")

x      <- Age_Sex_Geno_Trait_data[,3] - mean(Age_Sex_Geno_Trait_data[,3]) #centered age
id     <- Age_Sex_Geno_Trait_data[,1] 
nknots <- 15
temp   <- seq(min(x), max(x), length=nknots+2)  #check this
knots  <- temp[2:(length(temp)-1)]
m      <- length(x)

X1     <- cbind(rep(1,m), Age_Sex_Geno_Trait_data[,5], x )
X2     <- cbind(rep(1,m), Age_Sex_Geno_Trait_data[,5], x, x^2)
X3     <- cbind(rep(1,m), Age_Sex_Geno_Trait_data[,5], x, x^2, x^3)
XT     <- cbind(rep(1,m), Age_Sex_Geno_Trait_data[,5], mut( Age_Sex_Geno_Trait_data[,3] ) )
Z      <- outer(x, knots, "-")
Z      <- Z*(Z>0)

colnames(X1) <- c("intercept", "sex", "age_m")
colnames(X2) <- c("intercept", "sex", "age_m", "age_m^2")
colnames(X3) <- c("intercept", "sex", "age_m", "age_m^2", "age_m^3")
colnames(XT) <- c("intercept", "sex", "mut")
group       <- rep(1, nrow(Age_Sex_Geno_Trait_data))

###genotype and trait data ###
genotype    <- Age_Sex_Geno_Trait_data[, 4]
y = Age_Sex_Geno_Trait_data[, 6]

##################################################
### start from here to fit mixed effect models ###
##################################################
fit11 <- lme(y ~ -1 + X1 + genotype, 
               random =list(group = pdIdent(~-1+Z), 
                               id = ~1),
               correlation = corExp(form = ~ x | group/id),
               method="ML")  

fit12 <- lme(y ~ -1 + X1, 
               random =list(group = pdIdent(~-1+Z), 
                               id = ~1),
               correlation = corExp(form = ~ x | group/id),
               method="ML")
 
p_value_1   <- anova(fit11,fit12)[2,9]
p_value_1

#####
fit21 <- lme(y ~ -1 + X1 + genotype, 
               random =list(id = ~1),
               correlation = corExp(form = ~ x | id),
               method="ML")  

fit22 <- lme(y ~ -1 + X1, 
               random =list(id = ~1),
               correlation = corExp(form = ~ x | id),
               method="ML")

p_value_2   <- anova(fit21,fit22)[2,9]
p_value_2
   
#####
fit31 <- lme(y ~ -1 + X2 + genotype, 
               random =list(id = ~1),
               correlation = corExp(form = ~ x | id),
               method="ML")  

fit32 <- lme(y ~ -1 + X2, 
               random =list(id = ~1),
               correlation = corExp(form = ~ x | id),
               method="ML")
p_value_3   <- anova(fit31,fit32)[2,9]
p_value_3

#####
fit41 <- lme(y ~ -1 + X3 + genotype, 
               random =list(id = ~1),
               correlation = corExp(form = ~ x | id),
               method="ML")  

fit42 <- lme(y ~ -1 + X3, 
               random =list(id = ~1),
               correlation = corExp(form = ~ x | id),
               method="ML")

p_value_4   <- anova(fit41,fit42)[2,9]
p_value_4

### Fit the true model since the trait y is generated by the function mut ###
fitT1 <- lme(y ~ -1 + XT + genotype, 
               random =list(id = ~1),
               correlation = corExp(form = ~ x | id),
               method="ML")  

fitT2 <- lme(y ~ -1 + XT, 
               random =list(id = ~1),
               correlation = corExp(form = ~ x | id),
               method="ML")

p_value_T   <- anova(fitT1, fitT2)[2,9]
p_value_T 
